Method: Graph.union

Defined in:
lib/graph.rb

.union(*graphs) ⇒ Graph

Return a new Graph which is the union of every given graph. Each node of the union is in one or more given graph(s) (idem for edges). The last argument may be a hash of options.

Parameters:

  • options (Hash)

    a customizable set of options

Returns:

See Also:



33
34
35
# File 'lib/graph.rb', line 33

def Graph::union(*graphs)
    perform_graphs_group_op(*graphs, &:|)
end